@@ -1,4 +1,4 @@
-/* $Id: gucharmap-script-chapters.c,v 1.4 2004/02/02 01:28:09 nlevitt Exp $ */
+/* $Id: gucharmap-script-chapters.c,v 1.6 2005/09/08 13:35:59 behdad Exp $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -183,7 +183,11 @@ go_to_character (GucharmapChapters *chapters,
         {
           selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (parent->tree_view));
           if (!gtk_tree_selection_iter_is_selected (selection, &iter))
-            gtk_tree_selection_select_iter (selection, &iter);
+            {
+              GtkTreePath *path = gtk_tree_model_get_path (parent->tree_model, &iter);
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (parent->tree_view), path, NULL, FALSE);
+              gtk_tree_path_free (path);
+            }
           return TRUE;
         }
     }
@@ -205,7 +209,7 @@ gucharmap_script_chapters_class_init (GucharmapScriptChaptersClass *clazz)
 }
 
 GType 
-gucharmap_script_chapters_get_type ()
+gucharmap_script_chapters_get_type (void)
 {
   static GType t = 0;
 
@@ -232,7 +236,7 @@ gucharmap_script_chapters_get_type ()
 }
 
 GtkWidget * 
-gucharmap_script_chapters_new ()
+gucharmap_script_chapters_new (void)
 {
   return GTK_WIDGET (g_object_new (gucharmap_script_chapters_get_type (), NULL));
 }
